/**
 * EPlayer Feed Class
 */

(function() {

'use strict';
/// Default config
var defaults = {
  /** @type bool
   *   If true, the config will be pulled, and the default
   *   channel will be loaded, before 'ready' is fired.
   */
  preloadDefaultChannel: false,
  /** @type number
   *   Specify the number of videos to be preloaded, 
   *   when a channel is loaded.
   */
  preloadVideos: 0,
  /** @type number
   *    If true, an IMG elemens will be created for each channel,
   *    containing thumbnails, sponsor logos, and banners.
   */
  preloadChannelImages: false,
  /** @type string
   *  Select which video thumbnails should be preloaded.
   *  none|large|small|small large
   */
  preloadVideoThumbnails: 'none',
  /**
   * @type string
   *   ePlayer playerID
   */
  playerID: 'aj9al35syeqz1i4gz7h4zthy1'
};

var config = {};

/**
 * Creates an EPlayerFeed object and pulls the config
 *
 * Configuration is read from TS.Config.ePlayerFeed and 
 * TS.Config.feeds, prioritized in that order.
 *
 * @fires loading
 * @fires ready
 * @fires error
 */
TS.Modules.EPlayerFeed = function EPlayerFeed() {
  this.defaultChannel = null;
  this.config = null;

  // Apply config
  config = $.extend({}, defaults, TS.Config.EPlayerFeed || {});

  var urlRoot = 'http://xml.eplayer.performgroup.com/eplayer/';
  var channels = {};
  this.channels = channels;

  var Feed = TS.Modules.Feed;

  var configURL = urlRoot + 
        'config/{playerID}/?_fmt=jsonp&_clbk={callback}';

  // Pull config
  var configFeed = new Feed();
  var defaultChannelID;
  configFeed.url = configURL;

  configFeed.pull({ playerID: config.playerID });
  $(configFeed).on('ready', function(event, data) {
    try {
      defaultChannelID =  data.rawResponse.defaultChannel.id;
    } catch (e) {
      $.trace.error('Failed to load default channelID: ' + e);
    } finally {
      this.defaultChannel = new Channel(defaultChannelID,this);
      if (config.preloadDefaultChannel) {
        this.defaultChannel.pullMetadata();
        $(this.defaultChannel).on('load', function() {
          $(this).trigger('defaultChannelReady', {
            defaultChannel: this.defaultChannel,
            config: this.config
          });
          $(this).trigger('ready', {
            defaultChannel: this.defaultChannel,
            config: this.config
          });
        }.bind(this));
      } else {
        $(this).trigger('ready', {
          config: this.config
        });
      }
      $(this).trigger('configReady', {
        config: this.config
      });
      this.config = data.processedResponse;
    }
  }.bind(this));

  $(configFeed).on('error', function(event, data) {
    $.trace.error('EPlayerFeed: Failed to load config feed: ' + data.error + 
                  ', ' + data.httpError);
    $(this).trigger('error', { 
      error: 'EPlayerFeed: Failed to load config feed: ' + data.error + 
             ', ' + data.httpError
    });
  });

  this.pullStructure = function() {
    // Pull channel structure
    var structureFeed = new Feed();
    structureFeed.url = urlRoot + 
      'structure/{playerID}?_fmt=jsonp&_clbk={callback}';

    // Populate channels var: channels[category][]
    structureFeed.pull({ playerID: config.playerID });
    $(structureFeed).on('ready', function(event, data) {
      this.channels = channels = {};
      var categories = data.rawResponse[0].category;
      (categories.forEach === undefined ? [categories] : categories).
        forEach(function(e, i, a) {
        (e.channel.forEach === undefined ? [e.channel] : e.channel).
          forEach(function(e_, i_, a_) {
          var channel = new Channel(e_,this);
          channel.category = e.name;
          channels[e.name] ? channels[e.name].push(channel) :
            channels[e.name] = [channel];
        }.bind(this));
      }.bind(this));
      $(this).trigger('structureReady');
    }.bind(this));
    $(structureFeed).on('error', function(event, data) {
      $.trace.error('EPlayerFeed: Failed to load structure feed: ' + data.error + 
                    ', ' + data.httpError);
    });
    $(structureFeed).on('ready', function(event, data) {
    }.bind(this));
    return this;
  };

  if (TS.Config.DEBUG) {
    $(this).on('configReady ready defaultChannelReady structureReady', 
               function(event) {
      $.trace.info('EPlayerFeed' + ': ' + event.type);
    });
  }
};

function Channel(channelObject,parent) {
  TS.Modules.Feed.call(this);
  this.parentObject = parent;
  var FEED_PAGE_SIZE = 7;
  
  this.url = 'http://xml.eplayer.performgroup.com/eplayer/mrss/{playerID}/{channelID}/{from}-{to}' +
               '?_fmt=jsonp&_clbk={callback}';

  if (typeof channelObject === 'object') {
    // channel data from structure feed
    this.thumbnailURL = channelObject.thumbnail;
    this.title = channelObject.name;
    this.id = channelObject.id;

    // preload thumbnail
    if (config.preloadThumbnails) {
      this.thumbnailIMG = $('<img />').attr('src', this.thumbnailURL);
    }
  } else if (channelObject) {
    // id only
    this.id = channelObject;
  } else {
    throw new TypeError('The argument must be a structure feed channel ' +
                        'object, or a channelID.');
  }

  /**
   * Pull channel metadata (will pull the first 7 videos too)
   */
  this.pullMetadata = function() {
    // Load first videos and channel data
    this.pull();

    return this;
  }.bind(this);

  /**
   * Get videos from channel
   * @param {number} offset
   * @param {number} count
   */
  
  var pulledPages = [];
  var pagesToPull = [];
  var multiPullRequestDatas = [];
  this.pullVideos = function(offset, count) {

  	 this.requestData.from = 7;
  	 this.requestData.to = 12;
  	 
  	pagesToPull.splice(0);
  	multiPullRequestDatas.splice(0);
  	
  	 for(var i = offset; i<offset + count; i++){
  	 	var pageOfElem = Math.ceil(i / 6); 
  	 	if(pulledPages.indexOf(pageOfElem) == -1 && pagesToPull.indexOf(pageOfElem) == -1){
  	 		var from = (pageOfElem  - 1) * 6 + 1;
  	 		var to = from + 5;
  	 		pagesToPull.push(pageOfElem);
  	 		multiPullRequestDatas.push({from:from,to:to});
  	 	}
  	 } 

  	 if(pagesToPull.length == 0){
  	 	$(this).trigger('pullVideosReady');
  	 }

  	this.multiPull(multiPullRequestDatas);	

    return this;
  }.bind(this);

  var videos = [];
  this.videos = videos;

$(this).on('multiPullReady',onMultiPullReady);
function onMultiPullReady(){
	$(this).trigger('pullVideosReady');
}

  $(this).on('ready', function(event, data) {
    loadVideos(data);
    if (!wait.count) {
      this.loaded = true;
    }

  });

  var loadVideos = function(data) {
    // load config on first pull
    if (!this.description) {
      $.extend(this, data.rawResponse.channel);
    }

    // check if we have this page this already
    if (pulledPages.indexOf(
        parseInt(data.rawResponse.channel.page.offset, 10)) !== -1) {
      return;
    }
    var videoItems = data.rawResponse.channel.item instanceof Array ?
      data.rawResponse.channel.item : [data.rawResponse.channel.item];

    videoItems.forEach(function(e, i, a) {
    	if(typeof e != "undefined"){
	      var video = new Video(e);
	      var videoIndex = (parseInt(data.rawResponse.channel.page.offset) - 1) * 6 + i;
	      videos[videoIndex] = video;
	      //videos.push(video);
	
	      if (!video.thumbnails.small.ready) {
	        wait();
	        $(video.thumbnails.small).on('load', ready);
	      }
	      if (!video.thumbnails.large.ready) {
	        wait();
	        $(video.thumbnails.large).on('load', ready);
	      }
     }
    });
    pulledPages.push(parseInt(data.rawResponse.channel.page.offset, 10));
    
  }.bind(this);

  this.requestData = {
    playerID: config.playerID,
    channelID: this.id,
    from: 1,
    to: 7
  };

  // Only fire ready, when all events we're waiting for are completed.
  var wait = function() {
    wait.count++;
  }.bind(this);
  var ready = function() {
    wait.count--;
    if (!wait.count) {
      $(this).trigger('load', { channel: this });
    }
  }.bind(this);
  wait.count = 0;
}

function Video(videoObject) {
	
		  this.title = videoObject.title;
		  this.description = videoObject.description;
		  this.date = videoObject.pubDate;
		  this.id = videoObject.guid['#text'];
		  this.streams = [];
		
		  var group = videoObject.group instanceof Array ? 
		    videoObject.group[0] : videoObject.group;
		  var streams = group.content instanceof Array ?
		    group.content : [group.content];
		
		  streams.forEach(function(e, i, a) {
		    this.streams.push(new Stream(e));
		  }.bind(this));
		
		  this.thumbnails = {
		    small: new Thumbnail(videoObject.thumbnail,
		     config.preloadVideoThumbnails.indexOf('small') !== -1),
		    large: new Thumbnail(group.thumbnail, 
		      config.preloadVideoThumbnails.indexOf('large') !== -1)
		  };
	
}

function Stream(streamObject) {
  this.bitrate = parseInt(streamObject.bitrate, 10);
  this.width = parseInt(streamObject.height, 10);
  this.height = parseInt(streamObject.height, 10);
  this.duration = parseInt(streamObject.duration, 10);
  this.url = streamObject.url;
}

function Thumbnail(thumbnailObject, preload) {
  this.width = parseInt(thumbnailObject.width, 10);
  this.height = parseInt(thumbnailObject.height, 10);
  this.url = thumbnailObject.url;

  if (preload) {
    this.img = $('<img />').
      attr('src', this.url).
      attr('width', this.width).
      attr('height', this.height)[0];
    $(this.img).on('load', function() {
      $(this).trigger('load', { image: this.img });
      this.ready = true;
    }.bind(this));
  } else {
    this.ready = true;
  }
}

$.extend(TS.Modules.EPlayerFeed, {
  moduleName: 'EPlayerFeed',
  autoInit: false
});
})();
