/**
 * @fileOverview videotracker.js VideoTracker - base script
 *
 * ## Usage ##
 *
 * Include this script first, than at least one traker and one
 * provider script, to do something useful. Right after the last included
 * script, call videoTracker.init(), optionally with an options object.
 *  
 *   <script src="js/omniture/videotracker.js"></script>
 *   <script src="js/omniture/videotracker-omniture.js"></script>
 *   <script src="js/omniture/videotracker-eventdispatcher.js"></script>
 *   <script>
 *     $(function()
 *     { 
 *       videoTracker.init(
 *       { 
 *         debug: true,
 *         omnitureTracker: { debug: true },
 *         eventdispatcherProvider: { debug: true },
 *       });
 *     });
 *   </script>
 *
 * ## Options ##
 *
 * You can pass options to this scipt in the options object,
 * and to the plugins, you need to add a <plugin-name>(Provider|Tracker)
 * field, and specify options inside that.
 *
 * See the 'defaults' field in the according scripts for available options.
 *
 * @author Andras Sevcsik <andras.sevcsik@performgroup.com>
 * @version 0.2
 * @requires jQuery >=1.7
 */

/*
 * Initialize everything on documentReady.
 */
jQuery(function($)
{
  /**
   * @namespace The global videoTracker object.
   */
  window.videoTracker =
  {
    /**
     * Initializes providers and trackers.
     *
     * On success, it emits a /ready/ event via /window/.
     *
     * @param {Object} options options
     * @see videoTracker.defaults for available options
     * @throws {Error} 
     */
    init: function(options_)
    {
      var vt = videoTracker;

      // override defaults
      var options = $.extend({}, vt.defaults, options_);
      vt.options = options;
      var debug = options.debug;

      this.log('videotracker: starting initialization.');

      // We will be waiting for *ready events.
      vt.waiting = 0;

      // bind success callback if available
      if (vt.options.success) 
        $(videoTracker).on('ready', options.success);
      
      // initialize activated trackers.
      if (options.trackers !== null)
      {
        for (key in options.trackers)
        {
          var name = options.trackers[key];
          if (name in vt.trackers)
          {
            // Need to wait for init.
            if (vt.trackers[name].asyncInit) vt.waiting++;
            vt.trackers[name].init(options[name + 'Tracker']);
          }
          else 
            throw Error('Tracker not registered: ' + name);
        }
      }
      else // if no trackers given, initialize all, for your pleasure
      {
        options.trackers = [];
        for (key in vt.trackers)
        {
          options.trackers.push(key);
          // Need to wait for init.
          if (vt.trackers[key].asyncInit) vt.waiting++;
          vt.trackers[key].init(options[key + 'Tracker']);
        }
      }
      
      // initialize activated providers.
      if (options.providers !== null)
      {
        for (key in options.providers)
        {
          var provider_name = options.providers[key];
          if (provider_name in vt.providers)
          {
            // Need to wait for init.
            if (vt.providers[provider_name].asyncInit) vt.waiting++;

            vt.providers[provider_name].init(options[provider_name + 'Provider']);
          }
          else 
            throw Error('Tracker not registered: ' + provider_name);
        }
      }
      else // if no providers given, initialize all, for your pleasure
      {
        options.providers = [];
        for (key in vt.providers)
        {
          options.providers.push(key);

          // Need to wait for init.
          if (vt.providers[key].asyncInit) vt.waiting++;

          vt.providers[key].init(options[key + 'Provider']);
        }
      }

      this.log('waiting for ' + vt.waiting + ' items to be ready.');

      /**
       * Handles when an async init is complete.
       *
       * Bound to /trackerready/ and /providerready/
       * on /videoTracker/.
       *
       * @event
       * @param {Event}
       */
      function onPluginReady(event)
      {
        videoTracker.log(event.type + ' received.');
        
        // Everyone is ready \o/
        vt.waiting--;
        if (!vt.waiting)
        {
          $(videoTracker).trigger('ready');
          videoTracker.log('ready.');
        }
      }

      $(videoTracker).on('trackerready', onPluginReady);
      $(videoTracker).on('providerready', onPluginReady);

      $(videoTracker).on('config', function(event, eventData)
      {
        videoTracker.log('config recevied, data:');
        videoTracker.log(event, eventData);
        // Update configuration
        $.extend(true, videoTracker.options, eventData);
      });
    },

    /**
     * Holds Tracker objects, by name.
     *
     * @type {Object}
     */
    trackers: {},

    /**
     * Register a tracker.
     *
     * @param {videoTracker.Tracker} tracker le tracker.
     */
    registerTracker: function(tracker)
    {
      if (tracker.name in videoTracker.trackers)
        throw new Error('Tracker "' + tracker.name + '" already registered.');

      videoTracker.trackers[tracker.name] = tracker;
    },

    /**
     * Holds Provider objects, by name.
     * 
     * @type Object
     */
    providers: {},

    /**
     * Register a provider.
     *
     * @param {videoTracker.Provider} provider le provider.
     */
    registerProvider: function(provider)
    {
      if (provider.name in videoTracker.providers)
        throw new Error('Provider "' + provider.name + '" already registered.');

      videoTracker.providers[provider.name] = provider;
    },

    /**
     * Log messages or throw exception. 
     * Used inernally by Provider.log and Tracker.log
     *
     * @param {string} message message
     * @param {string} [type='message'] error type, message|warning|error|exception. On 
     *   'exception', an exception will be thrown.
     * @param {string} [name='core'] name of the module
     * @throws Error
     */
    log: function(message, type, name) {
      
      if (type === undefined) {
        type = 'message';
      }

      if (name === undefined) {
        name = 'core';
      }

      // Only display messages and warnings in debug mode.
      if (!this.options.debug && 
          (type === 'message' || type === 'warning')) {
        return;
      }

      var prefix = 'videotracker-' + name + ' ' + type + ': ';

      // Call user-defined log function, and return if it returns false, 
      // do nothing
      if (this.options.logFunction && 
          this.options.logFunction(prefix + message) !== false) {
        switch (type) {
          case 'warning':
            console.warn(prefix + message);
            break;
          case 'error':
            console.error(prefix + message);
            break;
          case 'exception':
            throw new Error(prefix + message);
            break;
          default:
          case 'message':
            console.log(prefix + message);
            break;
        }
      }
    },

    /**
     * Default configuration values
     * @type {Object}
     */
    defaults:
    {
      /**
       * VideoTracker binds this to /ready/
       * @event
       * @type Function
       * @default null
       */
      success: null,
      /**
       * Print tons of messages on console.
       * @type bool 
       * @default false
       */
      debug: false,
      /**
       * Array of tracker names to use, if omitted,
       * all registered trackers will be activated, for your pleasure.
       * @type Array
       * @default null
       */
      trackers: null,
      /**
       * This function will be called on every log message.
       * @type function({string} message, {string} type) 
       *   /type/ can be: 'message', 'warning', 'error', 'exception'
       *   If the return value === false, the built-in console logging code
       *   will not run.
       */
      logFunction: null,
      /**
       * Array of provider names to use, if omitted,
       * all registered providers will be activated, for your pleasure.
       * @type Array 
       * @default null
       */
      providers: null,

      /**
       * Channel info
       * @type object
       */
      channel:
      {
        category: 'NoCategory',
        genre: 'NoGenre',
        id: 0,
        name: 'NoName'
      },

      /**
       * Language code (e.g. en_GB)
       * @type string
       */
      language: 'null_NULL',

      /**
       * Country code (e.g. uk)
       * @type string
       */
      country: 'null',

      /**
       * Player data
       * @type object,
       */
      player:
      {
        /** @type int */
        height: 0,
        /** @type int */
        width: 0,
        /** @type string */
        name: 'ePlayer',
        /** @type string */
        version: 'unknown',
        /** 'html' or 'flash' 
         * @type string */
        type: 'html',
        /** @type bool */
        embedded: true,
        /** @type bool */
        autoPlay: false
      },

      /** 
       * URL of the page. If the player is in an iframe,
       * the parent's url is the default.
       *
       * @type string 
       */
      siteUrl: parent.document.location.href,
      
      /**
       * Partner ID
       *
       * @type string
       */
      partnerID: '0'
    }
  };

  /**
   * Trackers should extend Tracker objects.
   *
   * @constructor
   * @param {string} name tracker identifier, used as a field name.
   * @see videoTracker.Tracker.prototype.name
   */
  videoTracker.Tracker = function(name) 
  {
    this.name = name;
    this.ready = false;
  };
  
  /**
   * Inializes the tracker. Will be called by videoTracker.init().
   *
   * If async is true, this should trigger /trackerready/ event 
   * on /window/.
   *
   * @param {Object} options user options for the tracker.
   */
  videoTracker.Tracker.prototype.init = function(options) 
  {
    if (videoTracker.options.debug) 
      this.log('the fastest tracker ever, is ready!');

    this.ready = true;
  };

  /**
   * Tells, that the init function is asynchronous,
   * and videoTracker should wait for an /trackerready/
   * event. False by default.
   *
   * @type {bool}
   */
  videoTracker.Tracker.prototype.asyncInit = false;

  /**
   * Tracker identifier. Must be a valid javascript variable name.
   *
   * @type {string}
   */
  videoTracker.Tracker.prototype.name = 'emptytracker';

  /**
   * Ready state. True, if init was successful.
   *
   * @type {bool}
   */
  videoTracker.Tracker.prototype.ready = false;

  /**
   * Logs a message to console.
   *
   * @param {string} message the error message.
   * @param {string} [type='message'] type of error. Valid values: 
   *   'message', 'warning', 'error', 'exception'
   */
  videoTracker.Tracker.prototype.log = function(message, type) {
    if (type === undefined) {
      type = 'message';
    }
    videoTracker.log(message, type, this.name);
  }

  /**
   * Providers should extend Provider objects.
   *
   * @constructor
   * @param {string} name provider identifier, used as field name.
   * @see videoTracker.Provider.prototype.name
   */
  videoTracker.Provider = function(name)
  {
    this.name = name;
    this.ready = false;
  };

  /**
   * Inializes the provider. Will be called by videoTracker.init().
   *
   * If asyncInit is true, this should trigger /providerready/ 
   * event on /window/.
   *
   * @param {Object} options user options for the provider.
   */
  videoTracker.Provider.prototype.init = function(options) 
  {
    if (videoTracker.options.debug) 
      this.log('the fastest provider ever, is ready!');

    this.ready = true;
    $(videoTracker).trigger('providerready', this);
  };

  /**
   * Tells, that the init function is asynchronous,
   * and videoTracker should wait for an /providerready/
   * event. False by default.
   *
   * @type {bool}
   */
  videoTracker.Provider.prototype.asyncInit = false;

  /**
   * Provider identifier. Must be a valid javascript variable name.
   *
   * @type {string}
   */
  videoTracker.Provider.prototype.name = 'emptyprovider';

  /**
   * Ready state. True, if init was successful.
   *
   * @type {bool}
   */
  videoTracker.Provider.prototype.ready = false;

  /**
   * Logs a message to console.
   *
   * @param {string} message the error message.
   * @param {string} [type='message'] type of error. Valid values: 
   *   'message', 'warning', 'error', 'exception'
   */
  videoTracker.Provider.prototype.log = function(message, type) {
    if (type === undefined) {
      type = 'message';
    }
    videoTracker.log(message, type, this.name);
  }

  /**
   * Video object should be passed as data to video events.
   *
   * If you pass a videoItem object as the first argument, it copies
   * the data from it.
   *
   * @constructor
   * @param {string|Object} title video title or videoItem object
   * @param {string} [description] video description
   * @param {string} [guid] guid
   * @param {int} [duration] duration in seconds
   * @param {string} [url] stream url
   * @param {string} [adType] 'postroll' | 'preroll' | null (if it's not an ad)
   * @param {string} [channel] the channel name
   * @param {bool} [live] true if the stream is a live stream
   * @param {bool} [dontTrack] disable tracking (e.g. for ads)
   */
  videoTracker.Video = function(title, description, guid, duration, url, adType, 
                                channel, live, dontTrack)
  {
    if (typeof title == 'object')
    {
      var videoData = title;
      this.title = videoData.title;
      this.description = videoData.description;
      this.guid = videoData.guid;
      this.duration = videoData.media[0].duration;
      this.url = videoData.media[0].url;
      this.dontTrack = videoData.dontTrack;
      if (videoData.type == 'Ad')
        this.adType = videoData.preRoll ? 'preroll' : 'postroll';
      else this.adType = null;
      this.channel = videoData.channelName;
      this.live = videoData.isLive;
    }
    else
    {
      this.title = title;
      this.description = description;
      this.guid = guid;
      this.duration = duration;
      this.url = url;
      this.dontTrack = dontTrack;
      this.adType = adType;
      this.channel = channel;
      this.live = live;
    }

    if (this.live) {
      this.title = '[LIVE] ' + this.title;
    }
  };
});

